import * as THREE from 'three'
import CoordinateAxes from './coordinateAxes'
// import Renderer from '@/components/common/Renderer'

/**
 * This renderer monitors the host renderer's camera, and keeps a coordinate axes
 * the same direction as host renderer's
 */
export default class CoordinateAxesRenderer {
    hostRenderer = null
    coordinateAxes = null
    camera = null
    scene = null
    renderer = null
    ambientLight = null
    height = 600 // size of render area
    width = 600

    constructor(width, height, renderer) {
        this.width = width || this.width
        this.height = height || this.height
        this.init()
        // this.setHostRenderer(renderer)
    }

    init() {
        this.initRenderer()
        this.initScene()
        this.animate()
    }

    initRenderer() {
        this.renderer = new THREE.WebGLRenderer({
            antialias: true,
            alpha: true
        })
        this.renderer.setSize(this.width, this.height)
    }

    initScene() {
        this.scene = new THREE.Scene()
        this.scene.layer = 6
        // this.scene.background = new THREE.Color(0xebf2f7)

        this.camera = new THREE.OrthographicCamera(-1, 1, 1, -1, 0.1, 100)
        // this.camera.position.set(1.5, 1.5, 1.5)
        // this.camera.lookAt(0, 0, 0)
        this.scene.add(this.camera)

        this.ambientLight = new THREE.AmbientLight(0xcccccc, 1)
        this.scene.add(this.ambientLight)

        this.coordinateAxes = new CoordinateAxes()
        this.scene.add(this.coordinateAxes)
    }

    render() {
        if (this.renderer && this.scene && this.camera) {
            this.update()
            this.renderer.render(this.scene, this.camera)
        }
    }

    animate() {
        requestAnimationFrame(this.animate.bind(this))
        this.render()
    }

    setHostRenderer(renderer) {
        this.hostRenderer = renderer
        this.update()
    }

    update() {
        if (!this.hostRenderer || !this.hostRenderer.camera) {
            return
        }
        const camera = this.hostRenderer.camera
        if (camera) {
            const target = new THREE.Vector3()
            camera.getWorldDirection(target)
            const up = camera.up
            this.updateCameraDirection(target, up)
        }
    }

    /**
     * Update axes according to camera direction.
     * Camera's direction is the only input factor for this class. It always look at the origin.
     * @param direction
     */
    updateCameraDirection(direction, up) {
        if (!this.camera || !direction) {
            return
        }
        direction.normalize()
        const distanceFactor = 200 // keep camera a little farer, so it looks better
        const centerDelta = 0.3 // put the lookAt point to be in the first quadrant
        this.camera.position.set(-direction.x * distanceFactor + centerDelta, -direction.y * distanceFactor + centerDelta, -direction.z * distanceFactor + centerDelta)
        this.camera.lookAt(centerDelta, centerDelta, centerDelta) // it always looks at the origin
        this.camera.up = up
    }
}